# encoding:utf-8

from django import forms
from django.forms.widgets import ClearableFileInput, CheckboxInput
from django.utils.html import conditional_escape
from django.utils.safestring import mark_safe

from .utils import basename
from .models import Topic, Comment
from .widgets import TextareaWidget


class FormAdminTopic(forms.ModelForm):
	'''
	Form for topic cadmin
	'''
	class Meta:
		model = Topic
		exclude = ('slug', 'id_attachment')
		widgets = {
			'description': TextareaWidget,
		}


class FormAddTopic(forms.ModelForm):
	'''
	Form for create one new topic
	'''

	class Meta:
		model = Topic
		exclude = ('forum', "user", "slug", "date", "id_attachment", "moderate")
		widgets = {
			'description': TextareaWidget,
		}

	def __init__(self, *args, **kwargs):

		super(FormAddTopic, self).__init__(*args, **kwargs)
		class_css = 'form-control'

		for key in self.fields:
			if key != "attachment":
				self.fields[key].required = True
				self.fields[key].widget.attrs['class'] = class_css
			else:
				self.fields[key].required = False


class CustomClearableFileInput(ClearableFileInput):
	'''
	  Changes order fields
	'''
	template_with_initial = (
		'%(initial_text)s: <a href="%(initial_url)s">%(initial)s</a> '
		'%(clear_template)s<br />%(input_text)s: %(input)s'
	)
	template_with_clear = '<br>  <label for="%(clear_checkbox_id)s">%(clear_checkbox_label)s</label> %(clear)s'

	def render(self, name, value, attrs=None):
		substitutions = {
			'initial_text': self.initial_text,
			'input_text': self.input_text,
			'clear_template': '',
			'clear_checkbox_label': self.clear_checkbox_label,
		}
		template = '%(input)s'
		substitutions['input'] = super(ClearableFileInput, self).render(name, value, attrs)

		if self.is_initial(value):
			template = self.template_with_initial
			substitutions.update(self.get_template_substitution_values(value))

			values = self.get_template_substitution_values(value)
			initial = basename(values['initial'])

			if not self.is_required:
				checkbox_name = self.clear_checkbox_name(name)
				checkbox_id = self.clear_checkbox_id(checkbox_name)
				substitutions['clear_checkbox_name'] = conditional_escape(checkbox_name)
				substitutions['clear_checkbox_id'] = conditional_escape(checkbox_id)
				substitutions['clear'] = CheckboxInput().render(checkbox_name, False, attrs={'id': checkbox_id})
				substitutions['clear_template'] = self.template_with_clear % substitutions
				substitutions['initial'] = conditional_escape(initial)

		return mark_safe(template % substitutions)


class FormEditTopic(forms.ModelForm):
	'''
	Form for edit one new topic
	'''

	class Meta:
		model = Topic
		exclude = ('forum', "user", "slug", "date", "id_attachment", "moderate")
		widgets = {
			'description': TextareaWidget,
			'attachment': CustomClearableFileInput,
		}

	def __init__(self, *args, **kwargs):

		super(FormEditTopic, self).__init__(*args, **kwargs)
		class_css = 'form-control'

		for key in self.fields:
			if key != "attachment":
				self.fields[key].required = True
				self.fields[key].widget.attrs['class'] = class_css
			else:
				self.fields[key].required = False


class FormAddComment(forms.ModelForm):
	'''
	Form for add comment to topic
	'''
	class Meta:
		model = Comment
		fields = ['description']
		widgets = {
			'description': TextareaWidget,
		}

	def __init__(self, *args, **kwargs):

		super(FormAddComment, self).__init__(*args, **kwargs)

		for key in self.fields:
			if key == "description":
				self.fields[key].required = True
				self.fields[key].widget.attrs['style'] = "width: 100%"



import os

from django.conf import settings
from django.contrib.auth.models import User
from django.db import models
from django.shortcuts import get_object_or_404
from django.template import defaultfilters
from django.utils.encoding import python_2_unicode_compatible
from django.utils.crypto import get_random_string
from django.utils.translation import ugettext_lazy as _

from .validators import valid_extension

@python_2_unicode_compatible
class Category(models.Model):

	idcategory = models.AutoField(primary_key=True)
	name = models.CharField(max_length=80)
	position = models.IntegerField(blank=True, default=0)
	hidden = models.BooleanField(
		blank=False, null=False, default=False,
		help_text=_('If checked, this category will be visible only for staff')
	)

	class Meta(object):
		ordering = ['position']
		verbose_name = _('Category')
		verbose_name_plural = _('Categories')

	def __str__(self):
		return self.name


@python_2_unicode_compatible
class Forum(models.Model):

	idforum = models.AutoField(primary_key=True)
	category = models.ForeignKey(
		Category, related_name='forums',
		verbose_name=_('Category')
	)
	parent = models.ForeignKey(
		'self', related_name='child_forums', verbose_name=_('Parent forum'),
		blank=True, null=True
	)
	name = models.CharField(_('Name'), max_length=80, unique=True)
	position = models.IntegerField(_('Position'), blank=True, default=0)
	description = models.TextField(_('Description'), blank=True)
	moderators = models.ForeignKey(
		User, blank=True, null=True,
		verbose_name=_('Moderators')
	)
	date = models.DateTimeField(_('Date'), blank=True, null=True)
	topics_count = models.IntegerField(_('Topics count'), blank=True, default=0)
	hidden = models.BooleanField(
		_('Hidden'), blank=False, null=False, default=False
	)
	is_moderate = models.BooleanField(_('Check topics'), default=False)

	class Meta(object):
		ordering = ['category', 'position']
		verbose_name = _('Forum')
		verbose_name_plural = _('Forums')

	def __str__(self):
		return self.name

	def escape_html_description(obj):
		return obj.description
	escape_html_description.allow_tags = True

def generate_path(instance, filename):

	folder = ""
	folder = "forum_" + str(instance.forum_id)
	folder = folder + "_user_" + str(instance.user)
	folder = folder + "_topic_" + str(instance.id_attachment)
	return os.path.join("forum", folder, filename)


@python_2_unicode_compatible
class Topic(models.Model):

	idtopic = models.AutoField(primary_key=True)
	forum = models.ForeignKey(
		Forum, related_name='topic', verbose_name=_('Forum')
	)
	user = models.ForeignKey(User, related_name='Topic', verbose_name=_('User'))
	slug = models.SlugField(max_length=100)
	title = models.CharField(_('Title'), max_length=80)
	date = models.DateTimeField(_('Date'), blank=False, db_index=False)
	description = models.TextField(_('Description'), blank=False, null=False)
	id_attachment = models.CharField(max_length=200, null=True, blank=True)
	attachment = models.FileField(
		_('File'), blank=True, null=True, upload_to=generate_path,
		validators=[valid_extension]
	)
	moderate = models.BooleanField(_('Moderate'), default=False)

	class Meta(object):
		ordering = ['date', 'title', 'forum']
		verbose_name = _('Topic')
		verbose_name_plural = _('Topics')

	def __str__(self):
		return self.title

	def delete(self, *args, **kwargs):
		idtopic = self.idtopic
		forum = self.forum_id

		topic = get_object_or_404(Topic, idtopic=idtopic)

		folder = ""
		folder = "forum_" + str(forum)
		folder = folder + "_user_" + str(topic.user.username)
		folder = folder + "_topic_" + str(topic.id_attachment)
		path_folder = os.path.join("forum", folder)
		media_path = settings.MEDIA_ROOT
		path = media_path + "/" +  path_folder

		# Remove attachment if exists
		from .utils import remove_folder, exists_folder
		if exists_folder(path):
			remove_folder(path)

		Topic.objects.filter(idtopic=idtopic).delete()
		self.update_forum_topics(self.forum, "subtraction")

	def save(self, *args, **kwargs):

		if not self.idtopic:
			self.slug = defaultfilters.slugify(self.title)
			self.update_forum_topics(self.forum, "sum")

		self.generate_id_attachment(self.id_attachment)
		super(Topic, self).save(*args, **kwargs)

	def update_forum_topics(self, forum, action):

		f = Forum.objects.get(name=forum)
		tot_topics = f.topics_count
		if action == "sum":
			tot_topics = tot_topics + 1
		elif action == "subtraction":
			tot_topics = tot_topics - 1

		Forum.objects.filter(name=forum).update(
			topics_count=tot_topics
		)

	def generate_id_attachment(self, value):
		if not value:
			self.id_attachment = get_random_string(length=32)


@python_2_unicode_compatible
class Comment(models.Model):

	idcomment = models.AutoField(primary_key=True)
	topic = models.ForeignKey(
		Topic, related_name='comments', verbose_name=_('Topic')
	)
	user = models.ForeignKey(
		User, related_name='comments', verbose_name=_('User')
	)
	date = models.DateTimeField(_('Date'), blank=True, db_index=True)
	description = models.TextField(_('Description'), blank=True)

	class Meta(object):
		ordering = ['date']
		verbose_name = _('Comment')
		verbose_name_plural = _('Comments')

	def __str__(self):
		return self.date


@python_2_unicode_compatible
class Notification(models.Model):

	idnotification = models.AutoField(primary_key=True)
	idobject = models.IntegerField(default=0)
	iduser = models.IntegerField(default=0)
	is_topic = models.BooleanField(default=0)
	is_comment = models.BooleanField(default=0)
	is_view = models.BooleanField(default=0)
	date = models.DateTimeField(blank=True, db_index=True)

	class Meta(object):
		ordering = ['date']

	def __str__(self):
		return self.date



from django.conf.urls import include, url
from django.contrib.auth.decorators import login_required

from .views import (
	ForumsView, ForumView, TopicView, NewTopicView,
	EditTopicView, DeleteTopicView, NewCommentView,
	EditCommentView, DeleteCommentView, AllNotification,
	SetNotifications
)

urlpatterns = [
	# Url for django-hitcount
	url(r'hitcount/', include('hitcount.urls', namespace='hitcount')),

	# Url's Django-musette
	url(
		r'^forums/$', ForumsView.as_view(), name='forums'
	),
	url(
		r'^forum/(?P<forum>.+)/$', ForumView.as_view(), name='forum'
	),
	url(
		r'^topic/(?P<forum>.+)/(?P<slug>[-\w]+)/(?P<idtopic>\d+)/$',
		TopicView, name='topic'
	),
	url(
		r'^newtopic/(?P<forum>.+)/$', login_required(NewTopicView.as_view()),
		name='newtopic'
	),
	url(
		r'^edit_topic/(?P<forum>.+)/(?P<idtopic>\d+)/$',
		login_required(EditTopicView.as_view()), name='edittopic'
	),
	url(
		r'^delete_topic/(?P<forum>.+)/(?P<idtopic>\d+)/$',
		login_required(DeleteTopicView.as_view()), name='deletetopic'
	),
	url(
		r'^newcomment/(?P<forum>.+)/(?P<slug>[-\w]+)/(?P<idtopic>\d+)/$',
		login_required(NewCommentView.as_view()), name='newcomment'
	),
	url(
		r'^updatecomment/(?P<forum>.+)/(?P<slug>[-\w]+)/(?P<idtopic>\d+)/(?P<idcomment>\d+)/$',
		login_required(EditCommentView.as_view()), name='updatecomment'
	),
	url(
		r'^removecomment/(?P<forum>.+)/(?P<slug>[-\w]+)/(?P<idtopic>\d+)/(?P<idcomment>\d+)/$',
		login_required(DeleteCommentView.as_view()), name='removecomment'
	),
	url(
		r'^forum_all_notification/$', login_required(AllNotification),
		name='forum_all_notification'
	),
	url(
		r'^forum_set_notifications/$', login_required(SetNotifications),
		name='forum_set_notifications'
	),
]



from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _

def valid_extension(value):
	'''
	Function that valid extension
	when upload file in form
	'''
	if (not value.name.endswith('.png') and
		not value.name.endswith('.jpeg') and  
		not value.name.endswith('.gif') and
		not value.name.endswith('.bmp') and  
		not value.name.endswith('.jpg')):

		text = _("Files allowed")
		files = ".jpg, .jpeg, .png, .gif, .bmp"
		raise ValidationError(text + ': ' + files)


# -*- coding: UTF-8 -*-
import os
import shutil

from django.db.models import get_model
from django.conf import settings
from django.core.paginator import Paginator
from django.shortcuts import get_object_or_404
from django.utils import formats, timezone
from django.utils.translation import ugettext_lazy as _

from .models import Forum, Topic, Comment, Notification
from .settings import (
	APP_PROFILE, MODEL_PROFILE,
	URL_PROFILE, FIELD_PHOTO_PROFILE
)


def exists_folder(route):

	'''
	This method verify that exists
	folder in base to route
	'''
	if os.path.exists(route):
		return True
	else:
		return False


def remove_folder(route_folder):
	'''
		This method remove one folder
	'''
	try:
		shutil.rmtree(route_folder)
	except Exception:
		pass


def remove_file(route_file):

	'''
	This method remove one file
	in base to route and image
	'''
	if route_file != "" and not route_file is None:
		if os.path.exists(route_file):
			os.remove(route_file)


def get_folder_attachment(topic):
	'''
		This method return the path of one
		folder attachment for app forum
	'''
	folder = ""
	folder = "forum_" + str(topic.forum_id)
	folder = folder + "_user_" + str(topic.user.username)
	folder = folder + "_topic_" + str(topic.id_attachment)
	path_folder = os.path.join("forum", folder)
	media_path = settings.MEDIA_ROOT
	path = media_path + "/" +  path_folder

	return path


def remove_folder_attachment(idtopic):
	'''
		This method remove folder attachment
		and subtract one topic.
	'''
	# Subtract one topic
	topic = get_object_or_404(Topic, idtopic=idtopic)
	forum = get_object_or_404(Forum, name=topic.forum, hidden=False)
	tot = forum.topics_count
	tot = tot - 1
	Forum.objects.filter(name=topic.forum, hidden=False).update(
		topics_count=tot
	)

	path = get_folder_attachment(topic)

	# Remove attachment if exists
	if exists_folder(path):
		remove_folder(path)


def get_id_profile(iduser):
	'''
		This method return one id
		of model profile
	'''
	Profile = get_model(APP_PROFILE, MODEL_PROFILE)
	profile = get_object_or_404(Profile, iduser_id=iduser)

	return profile


def get_photo_profile(profile):

	'''
		This method return the photo
		of model profile id
	'''
	field_photo = getattr(profile, FIELD_PHOTO_PROFILE)
	return field_photo


def get_users_topic(topic, myuser):
	'''
		This method return all users
		of one topic, else my user
	'''
	comments = Comment.objects.filter(topic_id=topic.idtopic)
	lista_us = []
	for comment in comments:
		if comment.user_id != myuser:
			if not comment.user_id in lista_us:
				lista_us.append(comment.user_id)

	return lista_us


def get_notifications(iduser):
	'''
		This method return Notification
		of one user
	'''
	try:
		notif = Notification.objects.filter(
					iduser=iduser).order_by("-date")
	except Notification.DoesNotExist:
		notif = None

	return notif


def get_datetime_topic(date):
	'''
		This method return info
		one datetime for topic or
		notification
	'''
	flag = True
	now = timezone.now()
	# If is beteween 1 and 10 return days
	difference = (now - date).days

	# If is this days return hours
	if difference == 0:
		flag = False
		diff = now - date
		minutes = (diff.seconds//60)%60
		hours = diff.seconds//3600
		if minutes < 60 and hours == 0:
			difference =  "%s %s" % (str((diff.seconds//60)%60) + "m ", _("ago"))
		else:
			difference =  "%s %s" % (str(diff.seconds//3600) + "h ", _("ago"))

	# If > 10 days return string date
	if difference > 1 and difference <= 10:
		flag = False
		difference = formats.date_format(date, "SHORT_DATETIME_FORMAT")

	if flag:
		difference = "%s %s" % (str(difference),  _("days ago"))

	return difference


def basename(value):
	'''
		This method return basename
		of one path
	'''
	return os.path.basename(value)


def helper_paginator(self, request, model, tot_record, nonRecPag):

	'''
	This function is responsible of Pagination
	'''
	result_list = Paginator(model, tot_record)
	try:
		page = int(request.GET.get('page'));
	except:
		page = 1

	if page <= 0:
		page = 1

	if(page > result_list.num_pages):
		page = result_list.num_pages

	if (result_list.num_pages >= page):
		pagina = result_list.page(page)
		Contexto = {
			nonRecPag: pagina.object_list,
			'page': page,
			'pages': result_list.num_pages,
			'has_next': pagina.has_next(),
			'has_prev': pagina.has_previous(),
			'next_page': page+1,
			'prev_page': page-1,
			'firstPage': 1,
		}
		return Contexto


def get_route_file(file_path, file_name):
	'''
		This method build the path for a file MEDIA
	'''
	try:
		route_file = file_path + "/" + file_name
	except Exception:
		route_file = ""

	return route_file



from django.db import router
from django.contrib import admin, messages
from django.contrib.admin import helpers
from django.contrib.admin.utils import get_deleted_objects
from django.core.exceptions import PermissionDenied
from django.template.response import TemplateResponse
from django.utils.encoding import force_text
from django.utils.translation import ugettext_lazy as _

from .forms import FormAdminTopic
from .models import Category, Forum, Topic, Comment

from .utils import remove_folder_attachment


class TopicAdmin(admin.ModelAdmin):
	form = FormAdminTopic
	list_display = ('title', 'forum', 'date')
	list_filter = ['title', 'date']
	search_fields = ['title']
	actions=['delete_topic']

	def get_actions(self, request):
		actions = super(TopicAdmin, self).get_actions(request)
		if 'delete_selected' in actions:
			del actions['delete_selected']
		return actions

	def delete_topic(self, request, queryset):
		'''
			This method remove topic's selected
			in the admin django. Can remove one
			o more records.
		'''
		if not self.has_delete_permission(request):
			raise PermissionDenied

		if request.POST.get("post"):
			for obj in queryset:
				idtopic = obj.idtopic
				#Remove folder attachment
				remove_folder_attachment(idtopic)
				# Delete record
				Topic.objects.filter(idtopic=idtopic).delete()

			n = queryset.count()
			self.message_user(request, _("Successfully deleted %(count)d record/s.") % {
	                "count": n, }, messages.SUCCESS)

			return None
		else:

			opts = self.model._meta

			if len(queryset) == 1:
				objects_name = force_text(opts.verbose_name)
			else:
				objects_name = force_text(opts.verbose_name_plural)

			using = router.db_for_write(self.model)

			deletable_objects, model_count, perms_needed, protected = get_deleted_objects(
				queryset, opts, request.user, self.admin_site, using)

			context = {
	            'title': "",
	            'delete_topic': [queryset],
	            'ids': queryset.values_list("idtopic"),
	            'action_checkbox_name': helpers.ACTION_CHECKBOX_NAME,
	            'opts': opts,
	            'objects_name': objects_name,
	            'deletable_objects': [deletable_objects],
        	}

			return TemplateResponse(
	        	request, 'musette/admin/confirm_delete.html',
	        	context, current_app=self.admin_site.name
	        )

	delete_topic.short_description = _("Delete selected %(verbose_name_plural)s")


class ForumAdmin(admin.ModelAdmin):
	list_display = ('name', 'category', 'escape_html_description', 'topics_count')
	list_filter = ['name', 'category']
	search_fields = ['name']

	# TinyMCE
	class Media:
		js = ('/static/js/libs/tiny_mce/tiny_mce.js', '/static/js/textareas.js')


admin.site.register(Category)
admin.site.register(Forum, ForumAdmin)
admin.site.register(Topic, TopicAdmin)
admin.site.register(Comment)



# -*- coding: UTF-8 -*-
import datetime

from django.conf import settings
from django.contrib import messages
from django.contrib.auth.models import User
from django.http import Http404, HttpResponse, HttpResponseRedirect
from django.shortcuts import render, get_object_or_404, redirect
from django.template import defaultfilters
from django.views.generic import View
from django.views.generic.edit import FormView
from django.utils.crypto import get_random_string
from django.utils.html import strip_tags

from django.utils.translation import ugettext_lazy as _

from endless_pagination.decorators import page_template
from log.utils import set_error_to_log

from .forms import FormAddTopic, FormEditTopic, FormAddComment
from .models import Category, Forum, Topic, Comment, Notification
from .settings import URL_PROFILE
from .utils import (
	remove_folder_attachment, get_id_profile,
	get_photo_profile, get_users_topic,
	get_notifications, remove_file,
	helper_paginator, get_route_file, remove_folder,
	exists_folder
)


class ForumsView(View):
	'''
	This view display all forum registered
	'''
	template_name = "musette/index.html"

	def get(self, request, *args, **kwargs):

		categories = Category.objects.filter(hidden=False)

		if request.user.id:
			notifications = get_notifications(request.user.id)
		else:
			notifications = None

		data = {
				'categories': categories,
				'notifications': notifications
			}

		return render(request, self.template_name, data)


class ForumView(View):
	'''
	This view display one forum registered
	'''
	template_name = "musette/forum.html"

	def get(self, request, forum, *args, **kwargs):

		forum = get_object_or_404(Forum, name=forum, hidden=False)
		topics = Topic.objects.filter(forum_id=forum.idforum)

		pag = helper_paginator(self, request, topics, 15, 'topics')

		if request.user.id:
			notifications = get_notifications(request.user.id)
		else:
			notifications = None

		data = {
			'forum': forum,
			'topics': pag['topics'],
			'paginator': pag,
			'notifications': notifications,
		}

		return render(request, self.template_name, data)


@page_template('musette/topic.html')
def TopicView(request, forum, slug, idtopic,
	template='musette/topic_index.html', extra_context=None,
	*args, **kwargs):
	'''
	This view display one Topic of forum
	'''

	forum = get_object_or_404(Forum, name=forum, hidden=False)
	topic = get_object_or_404(Topic, idtopic=idtopic, slug=slug)

	profile = get_id_profile(topic.user_id)
	field_photo = get_photo_profile(profile)

	form_comment = FormAddComment()

	comments = Comment.objects.filter(topic_id=idtopic)

	if request.user.id:
		notifications = get_notifications(request.user.id)
	else:
		notifications = None

	data = {
		'topic': topic,
		'profile': profile,
		'URL_PROFILE': URL_PROFILE,
		'field_photo': field_photo,
		'form_comment': form_comment,
		'comments': comments,
		'notifications': notifications,
	}

	if extra_context is not None:
		data.update(extra_context)
	return render(request, template, data)


class NewTopicView(FormView):
	'''
		This view allowed add new topic
	'''
	template_name = "musette/new_topic.html"
	form_class = FormAddTopic

	def get_success_url(self):
		return '/forum/' + self.kwargs['forum']

	def get(self, request, forum, *args, **kwargs):

		data = {
			'form': self.form_class,
			'forum': forum,
		}
		return render(request, self.template_name, data)

	def post(self, request, forum, *args, **kwargs):

		form = FormAddTopic(request.POST, request.FILES)

		if form.is_valid():
			obj = form.save(commit=False)

			now = datetime.datetime.now()
			user = User.objects.get(id=request.user.id)
			forum = get_object_or_404(Forum, name=forum)
			title = strip_tags(request.POST['title'])

			obj.date = now
			obj.user = user
			obj.forum = forum
			obj.title = title
			obj.slug = defaultfilters.slugify(request.POST['title'])

			if 'attachment' in request.FILES:
				id_attachment = get_random_string(length=32)
				obj.id_attachment = id_attachment

				file_name = request.FILES['attachment']
				obj.attachment = file_name

			if forum.is_moderate:
				if forum.moderators_id == request.user.id:
					obj.moderate = True
				else:
					obj.moderate = False
			else:
				obj.moderate = True

			obj.save()
			return self.form_valid(form, **kwargs)
		else:
			messages.error(request, _("Form invalid"))
			return self.form_invalid(form, **kwargs)


class EditTopicView(FormView):
	'''
		This view allowed edit topic
	'''
	template_name = "musette/edit_topic.html"
	form_class = FormEditTopic

	def get_success_url(self):
		return '/forum/' + self.kwargs['forum']

	def get(self, request, forum, idtopic, *args, **kwargs):

		topic = get_object_or_404(Topic, idtopic=idtopic, user_id=request.user.id)

		# Init fields form
		form = FormEditTopic(instance=topic)

		data = {
			'form': form,
			'topic': topic,
		}

		return render(request, self.template_name, data)

	def post(self, request, forum, idtopic, *args, **kwargs):

		topic = get_object_or_404(Topic, idtopic=idtopic, user_id=request.user.id)
		file_name = topic.attachment

		form = FormEditTopic(request.POST, request.FILES, instance=topic)
		file_path = settings.MEDIA_ROOT

		if form.is_valid():

			obj = form.save(commit=False)

			title = strip_tags(request.POST['title'])
			description = strip_tags(request.POST['description'])
			slug = defaultfilters.slugify(request.POST['title'])

			obj.title = title
			obj.description = description
			obj.slug = slug

			# If check field clear, remove file when update
			if 'attachment-clear' in request.POST:
				route_file = get_route_file(file_path, file_name.name)

				try:
					remove_file(route_file)
				except Exception:
					pass

			if 'attachment' in request.FILES:

				if not topic.id_attachment:
					id_attachment = get_random_string(length=32)
					obj.id_attachment = id_attachment

				file_name_post = request.FILES['attachment']
				obj.attachment = file_name_post

				# Route previous file
				route_file = get_route_file(file_path, file_name.name)

				try:
					# If a previous file exists it removed
					remove_file(route_file)
				except Exception:
					pass

			# Update topic
			form.save()

			return self.form_valid(form, **kwargs)
		else:
			messages.error(request, _("Form invalid"))
			return self.form_invalid(form, **kwargs)


class DeleteTopicView(View):
	'''
	This view will delete one topic
	'''
	def get(self, request, forum, idtopic, *args, **kwargs):

		# Previouly verify that exists the topic
		topic = get_object_or_404(Topic, idtopic=idtopic, user_id=request.user.id)

		iduser_topic = topic.user_id

		# If my user delete
		if request.user.id == iduser_topic:
			remove_folder_attachment(idtopic)
			Topic.objects.filter(idtopic=idtopic, user_id=iduser_topic).delete()
		else:
			error = ""
			error = error + 'The user ' + str(request.user.id)
			error = error + ' He is trying to remove the job ' + str(idtopic)
			error = error +	' of user ' + str(iduser_topic)

			set_error_to_log(request, error)
			raise Http404

		return redirect('forum', forum)


class NewCommentView(View):
	'''
		This view allowed add new comment to topic
	'''
	def get(self, request, forum, slug, idtopic, *args, **kwargs):
		raise Http404()

	def post(self, request, forum, slug, idtopic, *args, **kwargs):

		form = FormAddComment(request.POST)

		param = ""
		param = forum + "/" + slug
		param = param + "/" + str(idtopic) + "/"
		url = '/topic/' + param

		if form.is_valid():
			obj = form.save(commit=False)

			now = datetime.datetime.now()
			user = User.objects.get(id=request.user.id)
			topic = get_object_or_404(Topic, idtopic=idtopic)

			obj.date = now
			obj.user = user
			obj.topic_id = topic.idtopic

			inserted = obj.save()

			idcomment = obj.idcomment
			lista_us = get_users_topic(topic, request.user.id)
			for user in lista_us:
				notification = Notification(
								iduser=user, is_view=False,
								idobject=idcomment, date=now,
								is_topic=False, is_comment=True
							)
				notification.save()


			return HttpResponseRedirect(url)
		else:
			messages.error(request, _("Field required"))
			return HttpResponseRedirect(url)


class EditCommentView(View):
	'''
		This view allowed edit comment to topic
	'''
	def get(self, request, forum, slug, idtopic, idcomment, *args, **kwargs):
		raise Http404()

	def post(self, request, forum, slug, idtopic, idcomment, *args, **kwargs):

		param = ""
		param = forum + "/" + slug
		param = param + "/" + str(idtopic) + "/"
		url = '/topic/' + param

		description = request.POST.get('update_description')

		if description:

			iduser = request.user.id
			Comment.objects.filter(idcomment=idcomment, user=iduser).update(
				description=description
			)

			return HttpResponseRedirect(url)
		else:
			return HttpResponseRedirect(url)


class DeleteCommentView(View):
	'''
		This view allowed remove comment to topic
	'''
	def get(self, request, forum, slug, idtopic, idcomment, *args, **kwargs):
		raise Http404()

	def post(self, request, forum, slug, idtopic, idcomment, *args, **kwargs):

		param = ""
		param = forum + "/" + slug
		param = param + "/" + str(idtopic) + "/"
		url = '/topic/' + param

		try:
			iduser = request.user.id
			Comment.objects.filter(idcomment=idcomment, user=iduser).delete()

			return HttpResponseRedirect(url)
		except Exception:
			return HttpResponseRedirect(url)


@page_template('musette/all_notification.html')
def AllNotification(request, template='musette/all_notification_index.html',
					extra_context=None, *args, **kwargs):

	iduser = request.user.id

	Notification.objects.filter(iduser=iduser).update(is_view=True)

	notifications = get_notifications(iduser)
	data = {
		'notifications': notifications,
	}

	if extra_context is not None:
		data.update(extra_context)
	return render(request, template, data)


def SetNotifications(request):
	'''
		This view set all views notifications in true
	'''
	iduser = request.user.id
	Notification.objects.filter(iduser=iduser).update(is_view=True)

	return HttpResponse("Ok")



from django.conf import settings

APP_PROFILE = getattr(settings, 'APP_PROFILE', "profiles")
MODEL_PROFILE = getattr(settings, 'MODEL_PROFILE', "profile")
FIELD_PHOTO_PROFILE = getattr(settings, 'FIELD_PHOTO_PROFILE', "photo")

URL_PROFILE = getattr(settings, 'URL_PROFILE', "/profile/")


# encoding:utf-8
from django import template
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.contrib.staticfiles.templatetags.staticfiles import static
from django.utils.text import Truncator

from hitcount.models import HitCount

from ..models import Comment, Forum, Topic, Notification
from .photo import get_photo
from ..utils import (
	get_id_profile, get_photo_profile,
	get_datetime_topic
)

register = template.Library()


@register.filter
def in_category(category):
	'''
	This tag filter the forum for category
	'''
	return Forum.objects.filter(
		category_id=category,
		hidden=False
	)


@register.filter
def get_tot_comments(idtopic):
	'''
	This tag filter return the total
	comments of one topic
	'''
	return Comment.objects.filter(
		topic_id=idtopic,
	).count()


@register.simple_tag
def get_tot_views(idtopic):
	'''
	This tag filter return the total
	views for topic or forum
	'''
	try:
		content = Topic.objects.get(idtopic=idtopic)
		idobj = ContentType.objects.get_for_model(content)

		hit = HitCount.objects.get(
			object_pk=idtopic,
			content_type_id=idobj
		)
		total = hit.hits
	except Exception:
		total = 0

	return total


@register.filter
def get_tot_users_comments(topic):
	'''
	This tag filter return the total
	users of one topic
	'''
	idtopic = topic.idtopic
	users = Comment.objects.filter(topic_id=idtopic)

	data = ""
	lista = []
	for user in users:
		usuario = user.user.username
		if not usuario in lista:
			lista.append(usuario)

			photo = get_photo(user.user.id)

			tooltip = ""
			tooltip += "data-toggle='tooltip' data-placement='bottom'"
			tooltip += "title='"+ usuario +"'"
			data += "<a href='/profile/"+ usuario +"' "+tooltip+" >"
			data += "<img class='img-circle' src='"+str(photo)+"' "
			data += "width=30, height=30></a>"

	if len(users) == 0:
		usuario = topic.user.username
		iduser = topic.user.id

		photo = get_photo(iduser)
		tooltip = ""
		tooltip += "data-toggle='tooltip' data-placement='bottom'"
		tooltip += "title='"+ usuario +"'"
		data += "<a href='/profile/"+ usuario +"' "+tooltip+" >"
		data += "<img class='img-circle' src='"+str(photo)+"' "
		data += "width=30, height=30></a>"


	return data


@register.filter
def get_tot_topics_moderate(forum):
	'''
		This filter return info about
		Few topics missing for moderate
	'''
	topics_count = forum.topics_count
	idforum = forum.idforum

	moderates = Topic.objects.filter(
							forum_id=idforum,
							moderate=True).count()
	return topics_count - moderates


@register.filter
def get_item_notification(notification):
	'''
		This filter return info about
		one notification of one user
	'''
	idobject = notification.idobject
	is_comment = notification.is_comment
	is_topic = notification.is_comment

	html = ""
	if is_comment:
		try:
			comment = Comment.objects.get(idcomment=idobject)

			forum = comment.topic.forum.name
			slug = comment.topic.slug
			idtopic = comment.topic.idtopic

			description = Truncator(comment.description).chars(100)

			url_topic = "/topic/" + forum + "/" + slug + "/" + str(idtopic) + "/"
 			title = "<h5><a href='"+url_topic+"'><u>"+comment.topic.title+"</u></h5></a>"
			description = "<p>"+description+"</p>"

			name = comment.user.last_name + " " + comment.user.first_name
			username = comment.user.username

			profile = get_id_profile(comment.user.id)
			photo = get_photo_profile(profile)
			if photo:
				path_img = settings.MEDIA_URL + str(photo)
			else:
				path_img = static("img/profile.png")

			user = "<a href='/profile/"+username+"'><p>" +  name +"</p></a>"
			date = get_datetime_topic(notification.date)

			html += '<div class="list-group">'
			html += '   <div class="list-group-item">'
			html += '      <div class="row-action-primary">'
			html += '           <img src="'+path_img+'" width=30 height=30 class="img-circle" />'
			html += '       </div>'
			html += '       <div class="row-content">'
			html += '           <div class="least-content">'+date+'</div>'
			html += '           <h4 class="list-group-item-heading">'+title+'</h4>'
			html += '           <p class="list-group-item-text">'+description+'</p>'
			html += '           <p>'+user+'</p>'
			html += '        </div>'
			html += '   </div>'
			html += '   <div class="list-group-separator"></div>'
			html += '</div>'

		except Comment.DoesNotExist:
			html = ""
	else:
		html = ""

	return html


@register.filter
def get_pending_notifications(user):
	'''
		This method return total pending notifications
	'''
	return Notification.objects.filter(
				is_view=False, iduser=user).count()
